package org.hibernate.envers.test.integration.onetomany.compositekey;

import org.hibernate.envers.test.BaseEnversJPAFunctionalTestCase;
import org.hibernate.envers.test.Priority;
import org.hibernate.testing.TestForIssue;
import org.junit.Assert;
import org.junit.Test;

import javax.persistence.EntityManager;
import java.util.Arrays;

/**
 * @author Angel L. Villala&iacute;n-Garc&iacute;a (angel dot villalain at gmail dot com)
 */
@TestForIssue(jiraKey = "HHH-7625")
public class OneToManyCompositeKeyTest extends BaseEnversJPAFunctionalTestCase {
    private ManyToManyCompositeKey.ManyToManyId owning1Id = null;
    private ManyToManyCompositeKey.ManyToManyId owning2Id = null;

    @Override
    protected Class<?>[] getAnnotatedClasses() {
        return new Class[] { OneToManyOwned.class, ManyToManyCompositeKey.class, ManyToOneOwned.class };
    }

    @Test
    @Priority(10)
    public void initData() {
        EntityManager em = getEntityManager();

        OneToManyOwned oneToManyOwned = new OneToManyOwned("data", null);
        ManyToOneOwned manyToOneOwned1 = new ManyToOneOwned("data1");
        ManyToOneOwned manyToOneOwned2 = new ManyToOneOwned("data2");
        ManyToManyCompositeKey owning1 = new ManyToManyCompositeKey(oneToManyOwned, manyToOneOwned1);
        ManyToManyCompositeKey owning2 = new ManyToManyCompositeKey(oneToManyOwned, manyToOneOwned2);

        // Revision 1
        em.getTransaction().begin();
        em.persist(oneToManyOwned);
        em.persist(manyToOneOwned1);
        em.persist(manyToOneOwned2);
        em.persist(owning1);
        em.persist(owning2);
        em.getTransaction().commit();

        owning1Id = owning1.getId();
        owning2Id = owning2.getId();

        // Revision 2
        em.getTransaction().begin();
        owning1 = em.find(ManyToManyCompositeKey.class, owning1Id);
        em.remove(owning1);
        em.getTransaction().commit();

        // Revision 3
        em.getTransaction().begin();
        owning2 = em.find(ManyToManyCompositeKey.class, owning2Id);
        em.getTransaction().commit();
    }

    @Test
    public void testRevisionsCounts() {
        Assert.assertEquals(Arrays.asList(1, 2), getAuditReader().getRevisions(ManyToManyCompositeKey.class, owning1Id));
        Assert.assertEquals(Arrays.asList(1, 2), getAuditReader().getRevisions(ManyToManyCompositeKey.class, owning2Id));
    }
}
